--[[ NavHelper by MrInfernal  v1.1

GUI to improve usage of MQ2Nav and streamline travel
Allows the user to search by zone full name, short name, or ID and returns
a selectable table. Click a zone and you can Start, Pause, or Cancel travel.

Buttons to check for teleport locations near your selected zone and to cast the port spell.
]]

---@type Mq
local mq = require('mq')
---@type ImGui
local imgui = require 'ImGui'

-- Import the zone and port files
local zones = require('navzones')
DruPorts = require('druports')
WizPorts = require('wizports')

AllPorts = {}
for _, addwiz in ipairs(WizPorts) do
    table.insert(AllPorts, addwiz)
end
for _, adddru in ipairs(DruPorts) do
    table.insert(AllPorts, adddru)
end

-- Set default values
local zoneSearch = ""       -- when 'Search' is clicked
local results = {}          -- results from zoneLookup()
local portresults = {}      -- when 'Check' is clicked [Searches the zone for nearby ports]
local destZone = ""         -- the zone chosen after Searching
local portChecked = false   -- keep port results table from showing until data is available
local portSpell = ""        -- the port spell to cast
local groupTravel = true    -- If navigation commands should be send to DanNet group
local useCWTN = false       -- Disable if not using CWTN Druid
local characterChoices = {"Druid", "Wizard", "Both"}    -- Options for the settings combobox
ClassPorts = 1     -- Combobox selection (1: Dru, 2: Wiz, 3: Both")
local navPause = false      -- Toggles the Pause/Resume button for travelling

-- Add druids and wizards to a DanNet channel called porters
if mq.TLO.Me.Class == 'Druid' then
    mq.cmd('/squelch /dgae dru /djoin porters')
    mq.cmd('/squelch /dgae porters /dtell all Joining porters!')
elseif mq.TLO.Me.Class == 'Wizard' then
    mq.cmd('/squelch /dgae wiz /djoin porters')
else
    mq.cmd('/squelch /dge dru /djoin porters')
    mq.cmd('/squelch /dge wiz /djoin porters')
end

local function checkCWTN()
    if mq.TLO.Plugin('mq2druid').IsLoaded() then
        useCWTN = true
    else
        useCWTN = false
    end
end

-- takes the input field and searches through the navzones for matches
-- can match the full name, short name, or id
local function zoneLookup(input)
    local results = {}

    -- Convert the input to lowercase for case-insensitive comparison
    local lowerInput = string.lower(input)

    -- Checks for zones that match the input
    for _, zone in ipairs(zones) do
        local lowerName = string.lower(zone.name)
        local lowerShortName = string.lower(zone.shortname)

        if string.find(lowerName, lowerInput) or
           string.find(lowerShortName, lowerInput) or
           tostring(zone.id) == input then
            table.insert(results, zone)
        end
    end

    return results
end

-- checks the selected zone for nearby port spell locations
-- zoneports have the direct location of the spell, as well as the surrounding zones
-- Function to set ports based on classPorts
local function portCheck(input)
    local ports = {}
    local results = {}

    if ClassPorts == 1 then
        ports = DruPorts
    elseif ClassPorts == 2 then
        ports = WizPorts
    elseif ClassPorts == 3 then
        ports = AllPorts
    end

    for _, port in ipairs(ports) do
        if port.loc == input then
            table.insert(results, port)
        end
    end

    if (classPorts ~= 3 and #results < 2) or (classPorts == 3 and #results < 4) then
        for _, port in ipairs(ports) do
            if string.find(table.concat(port.connections,','), input) then
                table.insert(results, port)
            end
        end
    end

    return results
end

-- Create a function to handle the teleporting via slash command
local function boundPort(input)
    local found = false

    for _, port in ipairs(AllPorts) do
        if string.lower(port.name) == string.lower(input) then
            found = true
            break
        end
    end

    if found then
        if useCWTN then
            if mq.TLO.Me.Class == 'Druid' then
                mq.cmd('/squelch /dgae porters /dru pause on')
                mq.cmd('/dgt all Pausing MQ2Druid!')
            else
                mq.cmd('/squelch /dge porters /dru pause on')
                mq.cmd('/dgt all Pausing MQ2Druid!')
            end
        end
        if mq.TLO.Me.Class == 'Druid' then
            mq.cmdf('/squelch /dgae porters /casting "%s"', input)
            mq.cmdf('/dgt all Casting "%s"!', input)
        else
            mq.cmdf('/noparse /squelch /dge porters /docommand ${If[${Me.Book["%s"]}>0,/casting "%s",/dgt all Spell not known!]}', input, input)
            mq.cmdf('/timed 2 /noparse /squelch /dge porters /docommand ${If[${Cast.Status.Find[M]},/dgt all Memorizing "%s",/squelch none]}', input)
            mq.cmdf('/noparse /squelch /dge porters /timed 150 /dgt all Casting "%s" in ${Math.Calc[${Cast.Timing}/1000]} seconds', input)
            mq.cmd('/timed 240 /noparse /squelch /dge porters /dgt all ${Math.Calc[${Cast.Timing}/1000]} seconds left')
        end
    else
        mq.cmd('/dgt all Spell not found!')
    end
end

-- Commands to run after zoning
local function justZoned()
    if useCWTN then
        if mq.TLO.Me.Class == 'Druid' then
            mq.cmd('/squelch /dgae porters /dru pause off')
            mq.cmd('/dgt all Unpausing MQ2Druid!')
        else
            mq.cmd('/squelch /dge porters /dru pause off')
            mq.cmd('/dgt all Unpausing MQ2Druid!')
        end
    end
end

local function justGrouped(line)
    if groupTravel == false then
        groupTravel = true
    end
end
-------------- End of functions

-- main function to run the window
function navHelper(open)
    -- Set default position/size
    local main_viewport = imgui.GetMainViewport()
    imgui.SetNextWindowPos(main_viewport.WorkPos.x + 650, main_viewport.WorkPos.y + 20, ImGuiCond.FirstUseEver)
    imgui.SetNextWindowSize(1550, 850, ImGuiCond.FirstUseEver)

    local show = false
    open, show = imgui.Begin("NavHelper", open)

    if not show then
        ImGui.End()
        return open
    end

    ImGui.PushItemWidth(ImGui.GetFontSize() * -12);

    -- Beginning of window elements
    imgui.BeginTabBar('Main Bar', ImGuiTabBarFlags.Reorderable)
    if imgui.BeginTabItem('Main') then
        imgui.BeginChild('Main')
        imgui.Text("Search by zone name, short name, or ID")
        zoneSearch = ImGui.InputText('', zoneSearch)

        imgui.SameLine()
        if ImGui.Button("Search") then
            results = zoneLookup(zoneSearch)
            portChecked = false -- Clear the port info display
        end
        imgui.SameLine()
        if ImGui.Button("Reset") then
            results = {} -- Reset the results
            zoneSearch = "" -- Clear the search input
            portChecked = false -- Clear the port info display
        end

        imgui.Text('\n')

        --Buttons to Start, Pause, or Cancel travel
        imgui.Text('Travel: ')
        imgui.SameLine()
        if ImGui.Button("Start") then
            if destZone ~= "" then
                if groupTravel then
                    mq.cmd("/dgae /travelto ", destZone)
                    navPause = false
                else
                    mq.cmd("/travelto ", destZone)
                    navPause = false
                end
            else
                print('No zone selected!')
            end
        end
        imgui.SameLine()
        if not navPause then
            if ImGui.Button("Pause") then
                if groupTravel then
                    mq.cmd("/dgae /nav pause")
                    navPause = true
                else
                    mq.cmd("/nav pause")
                    navPause = true
                end
            end
        else
            if ImGui.Button("Resume") then
                if groupTravel then
                    mq.cmd("/dgae /nav pause")
                    navPause = false
                else
                    mq.cmd("/nav pause")
                    navPause = false
                end
            end
        end
        imgui.SameLine()
        if ImGui.Button("Stop") then
            if groupTravel then
                mq.cmd("/dgae /travelto stop")
                navPause = false
            else
                mq.cmd("/travelto stop")
                navPause = false
            end
        end 

        -- Buttons to Check for ports. Self Cast means the selected character will cast
        -- Group Cast will send a DanNet message to the porter channel
        -- Also pauses CWTN plugins

        imgui.SameLine(300)
        imgui.Text('Ports: ')
        imgui.SameLine()
        if ImGui.Button("Check") then
            if destZone ~= "" then
                portChecked = true
                portresults = portCheck(destZone)
            else
                print('No zone selected!')
            end
        end
        imgui.SameLine()
        if ImGui.Button("Cast") then
            if portSpell == "" then
                print ('No port location selected!')
            else
                boundPort(portSpell)
            end
        end

        ImGui.Separator()
        -- Create table headings
        imgui.Text('Zone Name')
        ImGui.SameLine(240)
        imgui.Text('Short Name')
        ImGui.SameLine(400)
        imgui.Text('Zone ID')
        imgui.Text('\n')

        -- Iterate over results and create a selectable row for each
        for _, result in ipairs(results) do
            local isSelected = destZone == result.shortname
            if ImGui.Selectable(result.name, isSelected, ImGuiSelectableFlags.SpanAllColumns) then
                -- Set the selected zone to pass to other functions
                destZone = result.shortname
            end

            -- Display short name in the second column
            ImGui.SameLine(240)
            ImGui.Text(result.shortname)

            -- Display ID in the third column
            ImGui.SameLine(400)
            ImGui.Text(tostring(result.id))
        end

        --Port results. Will only show when results are available
        if portChecked then
            imgui.Text('\n')
            ImGui.Separator()
            imgui.Text('Nearby port locations')
            imgui.Text('\n')
            -- Create table headings
            imgui.Text('Level')
            ImGui.SameLine(60)
            imgui.Text('Spell Name')
            ImGui.SameLine(260)
            imgui.Text('Location')
            ImGui.SameLine(380)
            imgui.Text('Target')
            ImGui.SameLine(460)
            imgui.Text('Class')
            imgui.Text('\n')

            -- Iterate over results and create a selectable row for each
            for _, result2 in ipairs(portresults) do
                if portresults == {} then
                    imgui.Text('No port locations found!')
                else
                    local label = tostring(result2.level) .. "           " .. result2.name
                    local isSelected2 = portSpell == result2.name
                    if ImGui.Selectable(label, isSelected2, ImGuiSelectableFlags.SpanAllColumns) then
                        portSpell = result2.name  -- Set the selected spell
                    end
                    
                    ImGui.SameLine(260)
                    imgui.Text(result2.loc)
                    ImGui.SameLine(380)
                    imgui.Text(result2.target)
                    ImGui.SameLine(460)
                    imgui.Text(result2.class)
                end
            end
        end
        imgui.EndChild()
        imgui.EndTabItem()
    end

    if imgui.BeginTabItem('Settings') then
        imgui.BeginChild('Settings')
        -- Add a checkbox and store its state in the 'groupTravel' variable [Default is true]
        groupTravel = imgui.Checkbox("Travel commands to group", groupTravel)
        ImGui.SameLine(260)
        imgui.Text("Select class ports to display:")
        ImGui.SameLine(450)
        ClassPorts = imgui.Combo("Select Character", ClassPorts, characterChoices, #characterChoices)
        imgui.Text("\n")
        useCWTN = imgui.Checkbox("Use CWTN Druid", useCWTN)

        imgui.EndChild()
        imgui.EndTabItem()
    end

    imgui.EndTabBar()
    imgui.Spacing()
    imgui.PopItemWidth()
    imgui.End()
    return open
end

-- Creates a slash command for porting. Allows creation of hotkeys
-- ex: /nh "circle of knowledge"
mq.bind("/nh", boundPort)

-- Unpauses the druid CWTN after zoning
mq.event('zoned', 'You have entered#*#', justZoned)
mq.event('grouped', 'You have joined the group#*#', justGrouped)

-- See if MQ2Druid is loaded. If so, then toggle the useCWTN box in settings/dru mode 0
checkCWTN()

local openGUI = true

ImGui.Register('NavHelper', function()
    openGUI = navHelper(openGUI)
end)

while openGUI do
    mq.doevents()
    mq.delay(1000) -- equivalent to '1s'
end